@@ -2,7 +2,7 @@
 ## ----------------------------------------------------------------
 ##
 ##   IGraph R package
-##   Copyright (C) 2005-2019  Gabor Csardi <csardi.gabor@gmail.com>
+##   Copyright (C) 2005-2014  Gabor Csardi <csardi.gabor@gmail.com>
 ##   334 Harvard street, Cambridge, MA 02139 USA
 ##
 ##   This program is free software; you can redistribute it and/or modify
@@ -69,7 +69,6 @@ graph.adjacency.dense <- function(adjmatrix, mode=c("directed", "undirected", "m
   res
 }
 
-
 m2triplet <- get0("mat2triplet", asNamespace("Matrix"), inherits=FALSE)
 if(!is.function(m2triplet)) ## <==> (packageVersion("Matrix") < "1.3")
     m2triplet <- function(x) {  ## a simplified version of new Matrix' mat2triplet()
@@ -79,9 +78,7 @@ if(!is.function(m2triplet)) ## <==> (packageVersion("Matrix") < "1.3")
         else list(i = T@i + 1L, j = T@j + 1L, x = T@x)
     }
 
-mat_summary <- function(x) as.data.frame(m2triplet(x))
-if(FALSE) ## should work, too, *faster* :
-  mat_summary <- function(x) do.call(cbind, m2triplet(x))
+mysummary <- function(x) do.call(cbind, m2triplet(x))
 
 graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "max",
                                                 "min", "upper", "lower", "plus"),
@@ -98,6 +95,8 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
     }
   }
 
+  mysummary <- Matrix::summary
+
   if (nrow(adjmatrix) != ncol(adjmatrix)) {
     stop("not a square matrix")
   }
@@ -107,13 +106,15 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
   ## to remove non-redundancies that can persist in a dgtMatrix
   if(inherits(adjmatrix, "dgTMatrix")) {
     adjmatrix = as(adjmatrix, "CsparseMatrix")
+  } else if (inherits(adjmatrix, "ddiMatrix")) {
+    adjmatrix = as(adjmatrix, "CsparseMatrix")
   }
 
   if (is.null(weighted) && mode=="undirected") { mode <- "max" }
 
   if (mode == "directed") {
     ## DIRECTED
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     if (!diag) { el <- el[ el[,1] != el[,2], ] }
   } else if (mode == "undirected") {
     ## UNDIRECTED, must be symmetric if weighted
@@ -125,10 +126,10 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
     } else {
       adjmatrix <- Matrix::tril(adjmatrix, -1)
     }
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
   } else if (mode=="max") {
     ## MAXIMUM
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     rm(adjmatrix)
     if (!diag) { el <- el[ el[,1] != el[,2], ] }
     el <- el[ el[,3] != 0, ]
@@ -158,7 +159,7 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
     } else {
       adjmatrix <- Matrix::triu(adjmatrix, 1)
     }
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     rm(adjmatrix)
     if (!diag) { el <- el[ el[,1] != el[,2], ] }
   } else if (mode=="lower") {
@@ -168,13 +169,13 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
     } else {
       adjmatrix <- Matrix::tril(adjmatrix, -1)
     }
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     rm(adjmatrix)
     if (!diag) { el <- el[ el[,1] != el[,2], ] }
   } else if (mode=="min") {
     ## MINIMUM
     adjmatrix <- sign(adjmatrix) * sign(Matrix::t(adjmatrix)) * adjmatrix
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     if (!diag) { el <- el[ el[,1] != el[,2], ] }
     el <- el[ el[,3] != 0, ]
     w <- el[,3]
@@ -204,7 +205,7 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
     } else {
       adjmatrix <- Matrix::tril(adjmatrix, -1)
     }
-    el <- mat_summary(adjmatrix)
+    el <- mysummary(adjmatrix)
     if (diag) {
       loop <- el[,1] == el[,2]
       el[loop,3] <- el[loop,3] / 2
@@ -309,10 +310,10 @@ graph.adjacency.sparse <- function(adjmatrix, mode=c("directed", "undirected", "
 #' @keywords graphs
 #' @examples
 #'
-#' adjm <- matrix(sample(0:1, 100, replace=TRUE, prob=c(0.9,0.1)), nc=10)
+#' adjm <- matrix(sample(0:1, 100, replace=TRUE, prob=c(0.9,0.1)), ncol=10)
 #' g1 <- graph_from_adjacency_matrix( adjm )
 #' adjm <- matrix(sample(0:5, 100, replace=TRUE,
-#'                       prob=c(0.9,0.02,0.02,0.02,0.02,0.02)), nc=10)
+#'                       prob=c(0.9,0.02,0.02,0.02,0.02,0.02)), ncol=10)
 #' g2 <- graph_from_adjacency_matrix(adjm, weighted=TRUE)
 #' E(g2)$weight
 #'
