@@ -39,9 +39,9 @@ get_es_ref <- get_vs_ref <- function(graph) {
 
 get_es_graph <- get_vs_graph <- function(seq) {
   at <- attr(seq, "env")
-  if (class(at) == "weakref") {
+  if (inherits(at, "weakref")) {
     weak_ref_key(at)$me
-  } else if (class(at) == "environment") {
+  } else if (inherits(at, "environment")) {
     get("graph", envir = at)
   } else {
     NULL
@@ -65,17 +65,28 @@ get_es_graph_id <- get_vs_graph_id <- function(seq) {
 
 #' Decide if two graphs are identical
 #'
+#' Two graphs are considered identical by this function if and only if
+#' they are represented in exactly the same way in the internal R
+#' representation. This means that the two graphs must have the same
+#' list of vertices and edges, in exactly the same order, with same
+#' directedness, and the two graphs must also have identical graph, vertex and
+#' edge attributes. 
+#'
 #' This is similar to \code{identical} in the \code{base} package,
-#' but ignores the mutable piece of igraph objects, that might be
-#' different, even if the two graphs are identical.
+#' but it ignores the mutable piece of igraph objects; those might be
+#' different even if the two graphs are identical.
 #' 
+#' Attribute comparison can be turned off with the \code{attrs} parameter if
+#' the attributes of the two graphs are allowed to be different.
+#'
 #' @param g1,g2 The two graphs
+#' @param attrs Whether to compare the attributes of the graphs
 #' @return Logical scalar
 #' @export
 
-identical_graphs <- function(g1, g2) {
+identical_graphs <- function(g1, g2, attrs=TRUE) {
   stopifnot(is_igraph(g1), is_igraph(g2))
-  .Call(C_R_igraph_identical_graphs, g1, g2)
+  .Call(C_R_igraph_identical_graphs, g1, g2, as.logical(attrs))
 }
 
 add_vses_graph_ref <- function(vses, graph) {
@@ -347,7 +358,7 @@ simple_vs_index <- function(x, i, na_ok = FALSE) {
 #' }
 #' 
 #' @section Vertex attributes:
-#' When indexing vertex sequences, vertex attributes can be refered
+#' When indexing vertex sequences, vertex attributes can be referred
 #' to simply by using their names. E.g. if a graph has a \code{name} vertex
 #' attribute, then \code{V(g)[name == "foo"]} is equivalent to
 #' \code{V(g)[V(g)$name == "foo"]}. See examples below.
@@ -358,7 +369,7 @@ simple_vs_index <- function(x, i, na_ok = FALSE) {
 #'   \item{\code{.nei}}{takes a vertex sequence as its argument
 #'     and selects neighbors of these vertices. An optional \code{mode}
 #'     argument can be used to select successors (\code{mode="out"}), or
-#'     precedessors (\code{mode="in"}) in directed graphs.}
+#'     predecessors (\code{mode="in"}) in directed graphs.}
 #'   \item{\code{.inc}}{Takes an edge sequence as an argument, and
 #'     selects vertices that have at least one incident edge in this
 #'     edge sequence.}
@@ -663,7 +674,7 @@ simple_es_index <- function(x, i) {
 #'   \item Named graphs can be indexed with character vectors,
 #'     to select edges with the given names. Note that a graph may
 #'     have edge names and vertex names, and both can be used to select
-#'     edges. Edge names are simply used as names of the nuumeric
+#'     edges. Edge names are simply used as names of the numeric
 #'     edge id vector. Vertex names effectively only work in graphs without
 #'     multiple edges, and must be separated with a \code{|} bar character
 #'     to select an edges that incident to the two given vertices. See
@@ -671,7 +682,7 @@ simple_es_index <- function(x, i) {
 #' }
 #'
 #' @section Edge attributes:
-#' When indexing edge sequences, edge attributes can be refered
+#' When indexing edge sequences, edge attributes can be referred
 #' to simply by using their names. E.g. if a graph has a \code{weight} edge
 #' attribute, then \code{E(G)[weight > 1]} selects all edges with a larger
 #' than one weight. See more examples below.
@@ -1223,7 +1234,7 @@ as.igraph.vs <- function(graph, v, na.ok=FALSE) {
   } else {
     if (is.logical(v)) {
       res <- as.vector(V(graph))[v]
-    } else if (is.numeric(v) && any(v<0)){
+    } else if (is.numeric(v) && any(v<0, na.rm=TRUE)){
       res <- as.vector(V(graph))[v]
     } else {
       res <- as.numeric(v)
